% Correction application Chapitre 4
% Macroconomie quantitative (2014, d. Ellipses) 
% par Christophe Cahn

% Ce fichier permet de reproduire les lements contenu
% dans le fichier App_4_VAR.pdf disponible sur le site
% www.christophecahn.fr/macroquant

% Filename : App_4_VAR.m
% Application : Matlab/Octave
% Date : 24-04-2014
% Christophe Cahn (c) 2014

addpath('../MMMtbx');

clear all; clc; close all;

%1.a
donnees = load('../data/ZEdata.txt');
t = donnees(:,1);
Q = donnees(:,2);
P = donnees(:,3);
r = donnees(:,4);

%1.b
pie = diff(log(P));

%1.c
r = (1+r(2:end)/100).^(1/4)-1;

%1.d
lY = log(Q(2:end));
[crap,y] = hptrend(lY,1600);

%1.e
t = t(2:end);
plot(t,r,'-r*',t,y,'-bd',t,pie,'-ks','MarkerSize',4);
legend('taux d''interet','production','inflation');
print('./App_4_fig_rypie.pdf','-dpdf')

%1.f
T = length(t);   % taille de l'chantillon
trend = (1:T)';
XX = [trend.^2 trend ones(T,1)];
YY = pie;
betp = (XX'*XX)\(XX'*YY) ;
disp('betp');
disp(betp);

%1.g
YY = r ;
betr = (XX'*XX)\(XX'*YY);
disp('betr');
disp(betr);

infl = pie - XX*betp ; 
rate = r - XX*betr ;

%1.h
figure;
subplot(2,2,1); plot(t,pie,'k',t,pie-infl,'r'); 
legend('serie originale','tendance'); title('inflation');
subplot(2,2,2); plot(t,r,'k',t,r-rate,'r'); 
legend('serie originale','tendance'); title('taux d''interet');
subplot(2,2,3); plot(t,infl); 
legend('cycle'); title('inflation');
subplot(2,2,4); plot(t,rate); 
legend('cycle'); title('taux d''interet');
print('./App_4_fig_inflrate.pdf','-dpdf')

%% Estimation d'un VAR

% 2.a
Y = [y infl rate];

% 2.b

p = 4 ;
var = rfvar3(Y,p,[],[],[],[]);
var.By 								% Coefficients du VAR (cf help rfvar3)


% 2.c
figure;
for ii = 1:3
    subplot(3,1,ii); plot(var.u(:,ii));
    title(['e_{' num2str(ii) ',t}']);
end;
print('./App_4_fig_e.pdf','-dpdf')

% 2.d

Sig_e = (var.u'*var.u)/(T-p-1) ;
disp('Sig_e =');
fprintf('\n');
disp(Sig_e);

disp(cov(var.u));

%% Fonctions de rponse
% 3.a

S = chol(Sig_e);

% 3.b
nstep = 60 ;
IRF = impulsdtrf(var.By,S,nstep);

% 3.c
figure;
lbl = {'production','inflation','taux d''interet'};
n = 1 ;                         % numro du graphe
for ii = 1:3                    % nombre de variables
    for jj = 1:3                % nombre de chocs     
        subplot(3,3,n); 
        tmp = IRF(ii,jj,:);
        plot((1:nstep),tmp(:),(1:nstep),zeros(1,nstep),':r');
        title(['\eta_{' int2str(jj) ',t}']);
        ylabel(char(lbl(ii)));
        n = n + 1;
    end
end
print('./App_4_fig_irf.pdf','-dpdf')

%% Intervales de confiance
% 4.a
residus = var.u ;
e_tmp = residus(randi(T-p,T-p,1),:);

% 4.b
Y_sim = zeros(size(Y));
Y_sim(1:p,:) = Y(1:p,:);
for ii = p+1:T
    for jj = 1:p
        Y_sim(ii,:) = Y_sim(ii,:) + Y_sim(ii-1,:)*var.By(:,:,jj)';      
    end
    Y_sim(ii,:) = Y_sim(ii,:) + e_tmp(ii-p,:);
end

% 4.c
var_tmp = rfvar3(Y_sim,p,[],[],[],[]);
Sig_e_tmp = (var_tmp.u'*var_tmp.u)/(T-p-1) ;
IRF_tmp = impulsdtrf(var_tmp.By,chol(Sig_e_tmp),nstep);

n = 1 ;                         % numro du graphe
for ii = 1:3                    % nombre de variables
    for jj = 1:3                % nombre de chocs     
        subplot(3,3,n); 
        tmp1 = IRF(ii,jj,:);
        tmp2 = IRF_tmp(ii,jj,:);
        plot((1:nstep),tmp1(:),'k',...
             (1:nstep),tmp2(:),'b',...
             (1:nstep),zeros(1,nstep),':r');
        title(['\eta_{' int2str(jj) ',t}']);
        ylabel(char(lbl(ii)));
        n = n + 1;
    end
end
print('./App_4_fig_2irf.pdf','-dpdf')

% 4.d
% initialisation
h = waitbar(0,'Please wait...');
nsim = 500 ;                       % d'aprs Hamilton, plutt 10000
BIRF = zeros(3,3,nstep,nsim);

for nn = 1:nsim
    waitbar(nn/nsim,h)
    e_tmp = var.u(randi(T-p,T-p,1),:);
    Y_sim = zeros(size(Y));
    Y_sim(1:p,:) = Y(1:p,:);
    for ii = p+1:T
        for jj = 1:p
            Y_sim(ii,:) = Y_sim(ii,:) + Y_sim(ii-jj,:)*var.By(:,:,jj)';
        end
        Y_sim(ii,:) = Y_sim(ii,:) + e_tmp(ii-p,:);
    end
    var_tmp = rfvar3(Y_sim,p,[],[],[],[]);
    Sig_e_tmp = (var_tmp.u'*var_tmp.u)/(T-p-1) ;
    BIRF(:,:,:,nn) = impulsdtrf(var_tmp.By,chol(Sig_e_tmp),nstep);
end
close(h);


for ii = 1:4
    tmp = BIRF(1,2,ii,:);
    subplot(2,2,ii); hist(tmp(:),100);
    title(['IRF a t+' int2str(ii)]);
end
print('./App_4_fig_distribirf.pdf','-dpdf')

tmp = BIRF(1,2,2,:);        % attention  la dimension de tmp
tmp = tmp(:);               % tmp est  prsent un vecteur colonne
[N,val] = hist(tmp,1000);
F = cumsum(N)/nsim;
figure
subplot(1,2,1); hist(tmp,1000); title('distribution');
subplot(1,2,2); plot(val,F); title('repartition');
print('./App_4_fig_hist.pdf','-dpdf')


irf_min = min(val(F>=.05));
irf_max = min(val(F>=.95));


disp('irf_min =');
fprintf('\n');
disp(irf_min);
fprintf('\n');
disp('irf_max =');
fprintf('\n');
disp(irf_max);
fprintf('\n');

IRF_min = zeros(3,3,nstep);
IRF_max = zeros(3,3,nstep);

for ii = 1:3
    for jj = 1:3
        for ss = 1:nstep
            tmp = BIRF(ii,jj,ss,:);
            tmp = tmp(:);               
            [N,val] = hist(tmp,1000);
            F = cumsum(N)/nsim;
            IRF_min(ii,jj,ss) = min(val(F>=.05));
            IRF_max(ii,jj,ss) = min(val(F>=.95));
        end
    end
end


figure;
lbl = {'production','inflation','taux d''interet'};
n = 1 ;                         % numro du graphe
for ii = 1:3                    % nombre de variables
    for jj = 1:3                % nombre de chocs     
        subplot(3,3,n); 
        tmp = IRF(ii,jj,:);
        tmp_min = IRF_min(ii,jj,:);
        tmp_max = IRF_max(ii,jj,:);
        hor = (1:nstep);
        plot(hor,tmp(:),...
             hor,tmp_min(:),'r',...
             hor,tmp_max(:),'r',...
             hor,zeros(1,nstep),':r');
        title(['\eta_{' int2str(jj) ',t}']);
        ylabel(char(lbl(ii)));
        n = n + 1;
    end
end

print('./App_4_fig_conf.pdf','-dpdf')